// drawobj.cpp - implementation for drawing objects
//
// This is a part of the Microsoft Foundation Classes C++ library.
// Copyright (C) 1992-1998 Microsoft Corporation
// All rights reserved.
//
// This source code is only intended as a supplement to the
// Microsoft Foundation Classes Reference and related
// electronic documentation provided with the library.
// See these sources for detailed information regarding the
// Microsoft Foundation Classes product.


#include "stdafx.h"
#include "drawcli.h"

#include "drawdoc.h"
#include "drawvw.h"
#include "drawobj.h"

#include "cntritem.h"
#include "rectdlg.h"

#include "HLinkFormatPage.h"
#include "HLinkPage.h"

IMPLEMENT_SERIAL(CDrawObj, CObject, 0)

CDrawObj::CDrawObj()
{
}

CDrawObj::~CDrawObj()
{
}

CDrawObj::CDrawObj(const CRect& position)
{
	m_position = position;
	m_pDocument = NULL;

	m_bPen = TRUE;
	m_logpen.lopnStyle = PS_INSIDEFRAME;
	m_logpen.lopnWidth.x = 1;
	m_logpen.lopnWidth.y = 1;
	m_logpen.lopnColor = RGB(0, 0, 0);

	m_bBrush = TRUE;
	m_logbrush.lbStyle = BS_SOLID;
	m_logbrush.lbColor = RGB(192, 192, 192);
	m_logbrush.lbHatch = HS_HORIZONTAL;
}

void CDrawObj::Serialize(CArchive& ar)
{
	CObject::Serialize(ar);
	if (ar.IsStoring())
	{
		ar << m_position;
		ar << (WORD)m_bPen;
		ar.Write(&m_logpen, sizeof(LOGPEN));
		ar << (WORD)m_bBrush;
		ar.Write(&m_logbrush, sizeof(LOGBRUSH));
	}
	else
	{
		// get the document back pointer from the archive
		m_pDocument = (CDrawDoc*)ar.m_pDocument;
		ASSERT_VALID(m_pDocument);
		ASSERT_KINDOF(CDrawDoc, m_pDocument);

		WORD wTemp;
		ar >> m_position;
		ar >> wTemp; m_bPen = (BOOL)wTemp;
		ar.Read(&m_logpen,sizeof(LOGPEN));
		ar >> wTemp; m_bBrush = (BOOL)wTemp;
		ar.Read(&m_logbrush, sizeof(LOGBRUSH));
	}
}

void CDrawObj::Remove()
{
	delete this;
}

void CDrawObj::Draw(CDC*)
{
}

void CDrawObj::DrawTracker(CDC* pDC, TrackerState state)
{
	ASSERT_VALID(this);

	switch (state)
	{
	case normal:
		break;

	case selected:
	case active:
		{
			int nHandleCount = GetHandleCount();
			for (int nHandle = 1; nHandle <= nHandleCount; nHandle += 1)
			{
				CPoint handle = GetHandle(nHandle);
				pDC->PatBlt(handle.x - 3, handle.y - 3, 7, 7, DSTINVERT);
			}
		}
		break;
	}
}

// position is in logical
void CDrawObj::MoveTo(const CRect& position, CDrawView* pView)
{
	ASSERT_VALID(this);

	if (position == m_position)
		return;

	if (pView == NULL)
	{
		Invalidate();
		m_position = position;
		Invalidate();
	}
	else
	{
		pView->InvalObj(this);
		m_position = position;
		pView->InvalObj(this);
	}
	m_pDocument->SetModifiedFlag();
}

// Note: if bSelected, hit-codes start at one for the top-left
// and increment clockwise, 0 means no hit.
// If !bSelected, 0 = no hit, 1 = hit (anywhere)

// point is in logical coordinates
int CDrawObj::HitTest(CPoint point, CDrawView* pView, BOOL bSelected)
{
	ASSERT_VALID(this);
	ASSERT(pView != NULL);

	if (bSelected)
	{
		int nHandleCount = GetHandleCount();
		for (int nHandle = 1; nHandle <= nHandleCount; nHandle += 1)
		{
			// GetHandleRect returns in logical coords
			CRect rc = GetHandleRect(nHandle,pView);
			if (point.x >= rc.left && point.x < rc.right &&
				point.y <= rc.top && point.y > rc.bottom)
				return nHandle;
		}
	}
	else
	{
		if (point.x >= m_position.left && point.x < m_position.right &&
			point.y <= m_position.top && point.y > m_position.bottom)
			return 1;
	}
	return 0;
}

// rect must be in logical coordinates
BOOL CDrawObj::Intersects(const CRect& rect)
{
	ASSERT_VALID(this);

	CRect fixed = m_position;
	fixed.NormalizeRect();
	CRect rectT = rect;
	rectT.NormalizeRect();
	return !(rectT & fixed).IsRectEmpty();
}

int CDrawObj::GetHandleCount()
{
	ASSERT_VALID(this);
	return 8;
}

// returns logical coords of center of handle
CPoint CDrawObj::GetHandle(int nHandle)
{
	ASSERT_VALID(this);
	int x, y, xCenter, yCenter;

	// this gets the center regardless of left/right and top/bottom ordering
	xCenter = m_position.left + m_position.Width() / 2;
	yCenter = m_position.top + m_position.Height() / 2;

	switch (nHandle)
	{
	default:
		ASSERT(FALSE);

	case 1:
		x = m_position.left;
		y = m_position.top;
		break;

	case 2:
		x = xCenter;
		y = m_position.top;
		break;

	case 3:
		x = m_position.right;
		y = m_position.top;
		break;

	case 4:
		x = m_position.right;
		y = yCenter;
		break;

	case 5:
		x = m_position.right;
		y = m_position.bottom;
		break;

	case 6:
		x = xCenter;
		y = m_position.bottom;
		break;

	case 7:
		x = m_position.left;
		y = m_position.bottom;
		break;

	case 8:
		x = m_position.left;
		y = yCenter;
		break;
	}

	return CPoint(x, y);
}

// return rectange of handle in logical coords
CRect CDrawObj::GetHandleRect(int nHandleID, CDrawView* pView)
{
	ASSERT_VALID(this);
	ASSERT(pView != NULL);

	CRect rect;
	// get the center of the handle in logical coords
	CPoint point = GetHandle(nHandleID);
	// convert to client/device coords
	pView->DocToClient(point);
	// return CRect of handle in device coords
	rect.SetRect(point.x-3, point.y-3, point.x+3, point.y+3);
	pView->ClientToDoc(rect);

	return rect;
}

HCURSOR CDrawObj::GetHandleCursor(int nHandle)
{
	ASSERT_VALID(this);

	LPCTSTR id;
	switch (nHandle)
	{
	default:
		ASSERT(FALSE);

	case 1:
	case 5:
		id = IDC_SIZENWSE;
		break;

	case 2:
	case 6:
		id = IDC_SIZENS;
		break;

	case 3:
	case 7:
		id = IDC_SIZENESW;
		break;

	case 4:
	case 8:
		id = IDC_SIZEWE;
		break;
	}

	return AfxGetApp()->LoadStandardCursor(id);
}

// point must be in logical
void CDrawObj::MoveHandleTo(int nHandle, CPoint point, CDrawView* pView)
{
	ASSERT_VALID(this);

	CRect position = m_position;
	switch (nHandle)
	{
	default:
		ASSERT(FALSE);

	case 1:
		position.left = point.x;
		position.top = point.y;
		break;

	case 2:
		position.top = point.y;
		break;

	case 3:
		position.right = point.x;
		position.top = point.y;
		break;

	case 4:
		position.right = point.x;
		break;

	case 5:
		position.right = point.x;
		position.bottom = point.y;
		break;

	case 6:
		position.bottom = point.y;
		break;

	case 7:
		position.left = point.x;
		position.bottom = point.y;
		break;

	case 8:
		position.left = point.x;
		break;
	}

	MoveTo(position, pView);
}

void CDrawObj::Invalidate()
{
	ASSERT_VALID(this);
	m_pDocument->UpdateAllViews(NULL, HINT_UPDATE_DRAWOBJ, this);
}

CDrawObj* CDrawObj::Clone(CDrawDoc* pDoc)
{
	ASSERT_VALID(this);

	CDrawObj* pClone = new CDrawObj(m_position);
	pClone->m_bPen = m_bPen;
	pClone->m_logpen = m_logpen;
	pClone->m_bBrush = m_bBrush;
	pClone->m_logbrush = m_logbrush;
	ASSERT_VALID(pClone);

	if (pDoc != NULL)
		pDoc->Add(pClone);

	return pClone;
}

void CDrawObj::OnEditProperties()
{
	ASSERT_VALID(this);

	CPropertySheet sheet( _T("Shape Properties") );
	CRectDlg dlg;
	dlg.m_bNoFill = !m_bBrush;
	dlg.m_penSize = m_bPen ? m_logpen.lopnWidth.x : 0;
	sheet.AddPage( &dlg );

	if (sheet.DoModal() != IDOK)
		return;

	m_bBrush = !dlg.m_bNoFill;
	m_bPen = dlg.m_penSize > 0;
	if (m_bPen)
	{
		m_logpen.lopnWidth.x = dlg.m_penSize;
		m_logpen.lopnWidth.y = dlg.m_penSize;
	}

	Invalidate();
	m_pDocument->SetModifiedFlag();
}

void CDrawObj::OnOpen(CDrawView* /*pView*/ )
{
	OnEditProperties();
}

void CDrawObj::SetLineColor(COLORREF color)
{
	ASSERT_VALID(this);

	m_logpen.lopnColor = color;
	Invalidate();
	m_pDocument->SetModifiedFlag();
}

void CDrawObj::SetFillColor(COLORREF color)
{
	ASSERT_VALID(this);

	m_logbrush.lbColor = color;
	Invalidate();
	m_pDocument->SetModifiedFlag();
}

#ifdef _DEBUG
void CDrawObj::AssertValid()
{
	ASSERT(m_position.left <= m_position.right);
	ASSERT(m_position.bottom <= m_position.top);
}
#endif

////////////////////////////////////////////////////////////////////////////
// CDrawRect

IMPLEMENT_SERIAL(CDrawRect, CDrawObj, 0)

CDrawRect::CDrawRect()
{
}

CDrawRect::CDrawRect(const CRect& position)
	: CDrawObj(position)
{
	ASSERT_VALID(this);

	m_nShape = rectangle;
	m_roundness.x = 16;
	m_roundness.y = 16;
}

void CDrawRect::Serialize(CArchive& ar)
{
	ASSERT_VALID(this);

	CDrawObj::Serialize(ar);
	if (ar.IsStoring())
	{
		ar << (WORD) m_nShape;
		ar << m_roundness;
	}
	else
	{
		WORD wTemp;
		ar >> wTemp; m_nShape = (Shape)wTemp;
		ar >> m_roundness;
	}
}

void CDrawRect::Draw(CDC* pDC)
{
	ASSERT_VALID(this);

	CBrush brush;
	if (!brush.CreateBrushIndirect(&m_logbrush))
		return;
	CPen pen;
	if (!pen.CreatePenIndirect(&m_logpen))
		return;

	CBrush* pOldBrush;
	CPen* pOldPen;

	if (m_bBrush)
		pOldBrush = pDC->SelectObject(&brush);
	else
		pOldBrush = (CBrush*)pDC->SelectStockObject(NULL_BRUSH);

	if (m_bPen)
		pOldPen = pDC->SelectObject(&pen);
	else
		pOldPen = (CPen*)pDC->SelectStockObject(NULL_PEN);

	CRect rect = m_position;
	switch (m_nShape)
	{
	case rectangle:
		pDC->Rectangle(rect);
		break;

	case roundRectangle:
		pDC->RoundRect(rect, m_roundness);
		break;

	case ellipse:
		pDC->Ellipse(rect);
		break;

	case line:
		if (rect.top > rect.bottom)
		{
			rect.top -= m_logpen.lopnWidth.y / 2;
			rect.bottom += (m_logpen.lopnWidth.y + 1) / 2;
		}
		else
		{
			rect.top += (m_logpen.lopnWidth.y + 1) / 2;
			rect.bottom -= m_logpen.lopnWidth.y / 2;
		}

		if (rect.left > rect.right)
		{
			rect.left -= m_logpen.lopnWidth.x / 2;
			rect.right += (m_logpen.lopnWidth.x + 1) / 2;
		}
		else
		{
			rect.left += (m_logpen.lopnWidth.x + 1) / 2;
			rect.right -= m_logpen.lopnWidth.x / 2;
		}

		pDC->MoveTo(rect.TopLeft());
		pDC->LineTo(rect.BottomRight());
		break;
	}

	pDC->SelectObject(pOldBrush);
	pDC->SelectObject(pOldPen);
}


int CDrawRect::GetHandleCount()
{
	ASSERT_VALID(this);

	return m_nShape == line ? 2 :
		CDrawObj::GetHandleCount() + (m_nShape == roundRectangle);
}

// returns center of handle in logical coordinates
CPoint CDrawRect::GetHandle(int nHandle)
{
	ASSERT_VALID(this);

	if (m_nShape == line && nHandle == 2)
		nHandle = 5;
	else if (m_nShape == roundRectangle && nHandle == 9)
	{
		CRect rect = m_position;
		rect.NormalizeRect();
		CPoint point = rect.BottomRight();
		point.x -= m_roundness.x / 2;
		point.y -= m_roundness.y / 2;
		return point;
	}

	return CDrawObj::GetHandle(nHandle);
}

HCURSOR CDrawRect::GetHandleCursor(int nHandle)
{
	ASSERT_VALID(this);

	if (m_nShape == line && nHandle == 2)
		nHandle = 5;
	else if (m_nShape == roundRectangle && nHandle == 9)
		return AfxGetApp()->LoadStandardCursor(IDC_SIZEALL);
	return CDrawObj::GetHandleCursor(nHandle);
}

// point is in logical coordinates
void CDrawRect::MoveHandleTo(int nHandle, CPoint point, CDrawView* pView)
{
	ASSERT_VALID(this);

	if (m_nShape == line && nHandle == 2)
		nHandle = 5;
	else if (m_nShape == roundRectangle && nHandle == 9)
	{
		CRect rect = m_position;
		rect.NormalizeRect();
		if (point.x > rect.right - 1)
			point.x = rect.right - 1;
		else if (point.x < rect.left + rect.Width() / 2)
			point.x = rect.left + rect.Width() / 2;
		if (point.y > rect.bottom - 1)
			point.y = rect.bottom - 1;
		else if (point.y < rect.top + rect.Height() / 2)
			point.y = rect.top + rect.Height() / 2;
		m_roundness.x = 2 * (rect.right - point.x);
		m_roundness.y = 2 * (rect.bottom - point.y);
		m_pDocument->SetModifiedFlag();
		if (pView == NULL)
			Invalidate();
		else
			pView->InvalObj(this);
		return;
	}

	CDrawObj::MoveHandleTo(nHandle, point, pView);
}

// rect must be in logical coordinates
BOOL CDrawRect::Intersects(const CRect& rect)
{
	ASSERT_VALID(this);

	CRect rectT = rect;
	rectT.NormalizeRect();

	CRect fixed = m_position;
	fixed.NormalizeRect();
	if ((rectT & fixed).IsRectEmpty())
		return FALSE;

	CRgn rgn;
	switch (m_nShape)
	{
	case rectangle:
	case hlink:
		return TRUE;

	case roundRectangle:
		rgn.CreateRoundRectRgn(fixed.left, fixed.top, fixed.right, fixed.bottom,
			m_roundness.x, m_roundness.y);
		break;

	case ellipse:
		rgn.CreateEllipticRgnIndirect(fixed);
		break;

	case line:
		{
			int x = (m_logpen.lopnWidth.x + 5) / 2;
			int y = (m_logpen.lopnWidth.y + 5) / 2;
			POINT points[4];
			points[0].x = fixed.left;
			points[0].y = fixed.top;
			points[1].x = fixed.left;
			points[1].y = fixed.top;
			points[2].x = fixed.right;
			points[2].y = fixed.bottom;
			points[3].x = fixed.right;
			points[3].y = fixed.bottom;

			if (fixed.left < fixed.right)
			{
				points[0].x -= x;
				points[1].x += x;
				points[2].x += x;
				points[3].x -= x;
			}
			else
			{
				points[0].x += x;
				points[1].x -= x;
				points[2].x -= x;
				points[3].x += x;
			}

			if (fixed.top < fixed.bottom)
			{
				points[0].y -= y;
				points[1].y += y;
				points[2].y += y;
				points[3].y -= y;
			}
			else
			{
				points[0].y += y;
				points[1].y -= y;
				points[2].y -= y;
				points[3].y += y;
			}
			rgn.CreatePolygonRgn(points, 4, ALTERNATE);
		}
		break;
	}
	return rgn.RectInRegion(fixed);
}

CDrawObj* CDrawRect::Clone(CDrawDoc* pDoc)
{
	ASSERT_VALID(this);

	CDrawRect* pClone = new CDrawRect(m_position);
	pClone->m_bPen = m_bPen;
	pClone->m_logpen = m_logpen;
	pClone->m_bBrush = m_bBrush;
	pClone->m_logbrush = m_logbrush;
	pClone->m_nShape = m_nShape;
	pClone->m_roundness = m_roundness;
	ASSERT_VALID(pClone);

	if (pDoc != NULL)
		pDoc->Add(pClone);

	ASSERT_VALID(pClone);
	return pClone;
}

////////////////////////////////////////////////////////////////////////////
// CDrawPoly

IMPLEMENT_SERIAL(CDrawPoly, CDrawObj, 0)

CDrawPoly::CDrawPoly()
{
	m_points = NULL;
	m_nPoints = 0;
	m_nAllocPoints = 0;
}

CDrawPoly::CDrawPoly(const CRect& position)
	: CDrawObj(position)
{
	m_points = NULL;
	m_nPoints = 0;
	m_nAllocPoints = 0;
	m_bPen = TRUE;
	m_bBrush = FALSE;
}

CDrawPoly::~CDrawPoly()
{
	if (m_points != NULL)
		delete[] m_points;
}

void CDrawPoly::Serialize( CArchive& ar )
{
	int i;
	CDrawObj::Serialize( ar );
	if( ar.IsStoring() )
	{
		ar << (WORD) m_nPoints;
		ar << (WORD) m_nAllocPoints;
		for (i = 0;i< m_nPoints; i++)
			ar << m_points[i];
	}
	else
	{
		WORD wTemp;
		ar >> wTemp; m_nPoints = wTemp;
		ar >> wTemp; m_nAllocPoints = wTemp;
		m_points = new CPoint[m_nAllocPoints];
		for (i = 0;i < m_nPoints; i++)
			ar >> m_points[i];
	}
}

void CDrawPoly::Draw(CDC* pDC)
{
	ASSERT_VALID(this);

	CBrush brush;
	if (!brush.CreateBrushIndirect(&m_logbrush))
		return;
	CPen pen;
	if (!pen.CreatePenIndirect(&m_logpen))
		return;

	CBrush* pOldBrush;
	CPen* pOldPen;

	if (m_bBrush)
		pOldBrush = pDC->SelectObject(&brush);
	else
		pOldBrush = (CBrush*)pDC->SelectStockObject(NULL_BRUSH);

	if (m_bPen)
		pOldPen = pDC->SelectObject(&pen);
	else
		pOldPen = (CPen*)pDC->SelectStockObject(NULL_PEN);

	pDC->Polygon(m_points, m_nPoints);

	pDC->SelectObject(pOldBrush);
	pDC->SelectObject(pOldPen);
}

// position must be in logical coordinates
void CDrawPoly::MoveTo(const CRect& position, CDrawView* pView)
{
	ASSERT_VALID(this);
	if (position == m_position)
		return;

	if (pView == NULL)
		Invalidate();
	else
		pView->InvalObj(this);

	for (int i = 0; i < m_nPoints; i += 1)
	{
		m_points[i].x += position.left - m_position.left;
		m_points[i].y += position.top - m_position.top;
	}

	m_position = position;

	if (pView == NULL)
		Invalidate();
	else
		pView->InvalObj(this);
	m_pDocument->SetModifiedFlag();
}

int CDrawPoly::GetHandleCount()
{
	return m_nPoints;
}

CPoint CDrawPoly::GetHandle(int nHandle)
{
	ASSERT_VALID(this);

	ASSERT(nHandle >= 1 && nHandle <= m_nPoints);
	return m_points[nHandle - 1];
}

HCURSOR CDrawPoly::GetHandleCursor(int )
{
	return AfxGetApp()->LoadStandardCursor(IDC_ARROW);
}

// point is in logical coordinates
void CDrawPoly::MoveHandleTo(int nHandle, CPoint point, CDrawView* pView)
{
	ASSERT_VALID(this);
	ASSERT(nHandle >= 1 && nHandle <= m_nPoints);
	if (m_points[nHandle - 1] == point)
		return;

	m_points[nHandle - 1] = point;
	RecalcBounds(pView);

	if (pView == NULL)
		Invalidate();
	else
		pView->InvalObj(this);
	m_pDocument->SetModifiedFlag();
}

// rect must be in logical coordinates
BOOL CDrawPoly::Intersects(const CRect& rect)
{
	ASSERT_VALID(this);
	CRgn rgn;
	rgn.CreatePolygonRgn(m_points, m_nPoints, ALTERNATE);
	return rgn.RectInRegion(rect);
}

CDrawObj* CDrawPoly::Clone(CDrawDoc* pDoc)
{
	ASSERT_VALID(this);

	CDrawPoly* pClone = new CDrawPoly(m_position);
	pClone->m_bPen = m_bPen;
	pClone->m_logpen = m_logpen;
	pClone->m_bBrush = m_bBrush;
	pClone->m_logbrush = m_logbrush;
	pClone->m_points = new CPoint[m_nAllocPoints];
	memcpy(pClone->m_points, m_points, sizeof(CPoint) * m_nPoints);
	pClone->m_nAllocPoints = m_nAllocPoints;
	pClone->m_nPoints = m_nPoints;
	ASSERT_VALID(pClone);

	if (pDoc != NULL)
		pDoc->Add(pClone);

	ASSERT_VALID(pClone);
	return pClone;
}

// point is in logical coordinates
void CDrawPoly::AddPoint(const CPoint& point, CDrawView* pView)
{
	ASSERT_VALID(this);
	if (m_nPoints == m_nAllocPoints)
	{
		CPoint* newPoints = new CPoint[m_nAllocPoints + 10];
		if (m_points != NULL)
		{
			memcpy(newPoints, m_points, sizeof(CPoint) * m_nAllocPoints);
			delete[] m_points;
		}
		m_points = newPoints;
		m_nAllocPoints += 10;
	}

	if (m_nPoints == 0 || m_points[m_nPoints - 1] != point)
	{
		m_points[m_nPoints++] = point;
		if (!RecalcBounds(pView))
		{
			if (pView == NULL)
				Invalidate();
			else
				pView->InvalObj(this);
		}
		m_pDocument->SetModifiedFlag();
	}
}

BOOL CDrawPoly::RecalcBounds(CDrawView* pView)
{
	ASSERT_VALID(this);

	if (m_nPoints == 0)
		return FALSE;

	CRect bounds(m_points[0], CSize(0, 0));
	for (int i = 1; i < m_nPoints; ++i)
	{
		if (m_points[i].x < bounds.left)
			bounds.left = m_points[i].x;
		if (m_points[i].x > bounds.right)
			bounds.right = m_points[i].x;
		if (m_points[i].y < bounds.top)
			bounds.top = m_points[i].y;
		if (m_points[i].y > bounds.bottom)
			bounds.bottom = m_points[i].y;
	}

	if (bounds == m_position)
		return FALSE;

	if (pView == NULL)
		Invalidate();
	else
		pView->InvalObj(this);

	m_position = bounds;

	if (pView == NULL)
		Invalidate();
	else
		pView->InvalObj(this);

	return TRUE;
}

////////////////////////////////////////////////////////////////////////////

IMPLEMENT_SERIAL(CDrawOleObj, CDrawObj, 0)

BOOL CDrawOleObj::c_bShowItems = TRUE;

CDrawOleObj::CDrawOleObj() : m_extent(0,0)
{
	m_pClientItem = NULL;
}

CDrawOleObj::CDrawOleObj(const CRect& position)
	: CDrawObj(position), m_extent(0, 0)
{
	m_pClientItem = NULL;
}

CDrawOleObj::~CDrawOleObj()
{
	if (m_pClientItem != NULL)
	{
		m_pClientItem->Release();
		m_pClientItem = NULL;
	}
}

void CDrawOleObj::Remove()
{
	if (m_pClientItem != NULL)
	{
		m_pClientItem->Delete();
		m_pClientItem = NULL;
	}
	CDrawObj::Remove();
}

void CDrawOleObj::Serialize( CArchive& ar )
{
	ASSERT_VALID(this);

	CDrawObj::Serialize(ar);

	if (ar.IsStoring())
	{
		ar << m_extent;
		ar << m_pClientItem;
	}
	else
	{
		ar >> m_extent;
		ar >> m_pClientItem;
		m_pClientItem->m_pDrawObj = this;
	}
}

CDrawObj* CDrawOleObj::Clone(CDrawDoc* pDoc)
{
	ASSERT_VALID(this);

	AfxGetApp()->BeginWaitCursor();

	CDrawOleObj* pClone = NULL;
	CDrawItem* pItem = NULL;
	TRY
	{
		// perform a "deep copy" -- need to copy CDrawOleObj and the CDrawItem
		//  that it points to.
		CDrawOleObj* pClone = new CDrawOleObj(m_position);
		CDrawItem* pItem = new CDrawItem(m_pDocument, pClone);
		if (!pItem->CreateCloneFrom(m_pClientItem))
			AfxThrowMemoryException();

		pClone->m_pClientItem = pItem;
		pClone->m_bPen = m_bPen;
		pClone->m_logpen = m_logpen;
		pClone->m_bBrush = m_bBrush;
		pClone->m_logbrush = m_logbrush;
		ASSERT_VALID(pClone);

		if (pDoc != NULL)
			pDoc->Add(pClone);
	}
	CATCH_ALL(e)
	{
		pItem->Delete();
		pClone->m_pClientItem = NULL;
		pClone->Remove();
		AfxGetApp()->EndWaitCursor();

		THROW_LAST();
	}
	END_CATCH_ALL

	AfxGetApp()->EndWaitCursor();
	return pClone;
}

//AG begin
void CDrawOleObj::DrawTracker(CDC* pDC, TrackerState state)
{
	CDrawObj::DrawTracker(pDC,state);
	
	CDrawItem* pItem = m_pClientItem;
	if (pItem != NULL)
	{
	
		
		// use a CRectTracker to draw the standard effects
		CRectTracker tracker;
		tracker.m_rect = m_position;
		pDC->LPtoDP(tracker.m_rect);
		
		if (c_bShowItems)
		{
			// put correct border depending on item type
			if (pItem->GetType() == OT_LINK)
				tracker.m_nStyle |= CRectTracker::dottedLine;
			else
				tracker.m_nStyle |= CRectTracker::solidLine;
		}
		
		// put hatching over the item if it is currently open
		if (pItem->GetItemState() == COleClientItem::openState ||
			pItem->GetItemState() == COleClientItem::activeUIState)
		{
			tracker.m_nStyle |= CRectTracker::hatchInside;
		}
		tracker.Draw(pDC);
	}
}

void CDrawOleObj::Draw(CDC* pDC)
{
	ASSERT_VALID(this);

	CDrawItem* pItem = m_pClientItem;
	if (pItem != NULL)
	{
		// draw the OLE item itself
		pItem->Draw(pDC, m_position);
	}
}
//AG end

void CDrawOleObj::OnOpen(CDrawView* pView)
{
	AfxGetApp()->BeginWaitCursor();
	m_pClientItem->DoVerb(
		GetKeyState(VK_CONTROL) < 0 ? OLEIVERB_OPEN : OLEIVERB_PRIMARY,
		pView);
	AfxGetApp()->EndWaitCursor();
}

void CDrawOleObj::OnEditProperties()
{
	// using COlePropertiesDialog directly means no scaling
	COlePropertiesDialog dlg(m_pClientItem, 100, 100, NULL);

	dlg.DoModal();
}

// position is in logical
void CDrawOleObj::MoveTo(const CRect& position, CDrawView* pView)
{
	ASSERT_VALID(this);

	if (position == m_position)
		return;

	// call base class to update position
	CDrawObj::MoveTo(position, pView);

	// update position of in-place editing session on position change
	if (m_pClientItem->IsInPlaceActive())
		m_pClientItem->SetItemRects();
}

/////////////////////////////////////////////////////////////////////////////

//HLink begin
/////////////////////////////////////////////////////////////////////////////
CFontsList::CFontsList()	//Stores the default font in the list
{
	CFont* pFirstFont=new CFont;
	pFirstFont->CreateStockObject(ANSI_VAR_FONT);
	AddTail(pFirstFont);
	m_posDefaultFont=GetHeadPosition();
}


CFontsList::~CFontsList()
{
	POSITION pos=GetHeadPosition();
	while(pos)
	{
		CFont* pFont=GetNext(pos);
		delete pFont;
	}
}


POSITION CFontsList::GetFontPosIndirect(const LOGFONT* pLogFont)
{
	LOGFONT lfCurrent;
	POSITION pos=GetHeadPosition();
	while(pos)
	{
		POSITION posNow=pos;
		CFont* pFont=GetNext(pos);
		pFont->GetLogFont(&lfCurrent);
		if(memcmp(&lfCurrent,pLogFont,sizeof(LOGFONT))==0)
		{
			m_posDefaultFont=posNow;
			return posNow;
		}
	}

	//Font not found
	CFont* pNewFont=new CFont;
	pNewFont->CreateFontIndirect(pLogFont);
	return AddTail(pNewFont);
}



////////////////////////////////////////////////////////////////////////////
// CDrawHLink
IMPLEMENT_SERIAL(CDrawHLink, CDrawRect, 0)

CFontsList CDrawHLink::staticLoadedFontsList;
int CDrawHLink::nCurrentPropertyPage=0;

CDrawHLink::CDrawHLink()
{
	m_nShape=rectangle;
	m_posFont=staticLoadedFontsList.GetDefaultFontPos();
	m_iHorAlignment=DT_LEFT;
	m_iVerAlignment=DT_TOP;
	m_bSingleLine=FALSE;
	m_rgbTextColour=0;
	m_dwSiteData=0;
}

CDrawHLink::CDrawHLink(const CRect& position)
					:CDrawRect(position)
{
	m_nShape=rectangle;
	m_posFont=staticLoadedFontsList.GetDefaultFontPos();
	m_iHorAlignment=DT_LEFT;
	m_iVerAlignment=DT_TOP;
	m_bSingleLine=FALSE;
	m_rgbTextColour=0;
	m_dwSiteData=0;
}



// ReadObjectFromStreamFlat / WriteObjectToStreamFlat
// Utility funcs to persist OLE objects from IStream to raw bytes in archive
// Ssee COleClientItem::ReadItemFlat, WriteItemFlat for implementations of something
// similar for IStorage-based OLE docs. Hyperlinks are lightweight little guys and
// only support IStream.

void ReadObjectFromStreamFlat(CArchive& ar, LPUNKNOWN* lppUnkObject)
{
	HRESULT hr;

	// read number of bytes in the ILockBytes
	DWORD dwBytes;
	ar >> dwBytes;

	// allocate enough memory to read entire block
	HGLOBAL hStream = ::GlobalAlloc(GMEM_SHARE|GMEM_MOVEABLE, dwBytes);
	if (hStream == NULL)
		AfxThrowMemoryException();

	LPVOID lpBuf = ::GlobalLock(hStream);
	ASSERT(lpBuf != NULL);
	DWORD dwBytesRead = ar.Read(lpBuf, dwBytes);
	::GlobalUnlock(hStream);

	// throw exception in case of partial object
	if (dwBytesRead != dwBytes)
	{
		::GlobalFree(hStream);
		AfxThrowArchiveException(CArchiveException::endOfFile);
	}

	IStreamPtr pStream;
	hr = ::CreateStreamOnHGlobal(hStream, true, &pStream);
	if (FAILED(hr))
	{
		::GlobalFree(hStream);
		AfxThrowMemoryException();
	}

	// attempt to load the object from the storage
	LPUNKNOWN lpUnk = NULL;
	hr = ::OleLoadFromStream(pStream, IID_IUnknown, (void**)&lpUnk);
	if (FAILED(hr))
		*lppUnkObject = NULL;
	else
		*lppUnkObject = lpUnk;
}

void WriteObjectToStreamFlat(CArchive& ar, IUnknownPtr lpUnkObject)
{
	HRESULT hr;

	IStreamPtr pStream;
	hr = ::CreateStreamOnHGlobal(NULL, true, &pStream);
	if (FAILED(hr))
		AfxThrowMemoryException();

	// Write object to stream
	IPersistStreamPtr pPersistStream = lpUnkObject;
	if (NULL == pPersistStream)
		AfxThrowOleException(E_NOINTERFACE);

	hr = ::OleSaveToStream(pPersistStream, pStream);
	if (FAILED(hr))
		AfxThrowOleException(hr);

	// attempt to get the handle to the global memory
	HGLOBAL hStream;
	hr = ::GetHGlobalFromStream(pStream, &hStream);
	if (FAILED(hr))
		AfxThrowOleException(hr);

	// first write a byte count
	DWORD dwBytes;
	if (!(dwBytes = ::GlobalSize(hStream)))
		AfxThrowMemoryException();
	ar << dwBytes;

	// write the contents of the block
	LPVOID lpBuf = ::GlobalLock(hStream);
	ASSERT(lpBuf != NULL);
	ar.Write(lpBuf, (UINT)dwBytes);
	::GlobalUnlock(hStream);
}


void CDrawHLink::Serialize(CArchive& ar)
{
	LOGFONT lfFont;
	CDrawRect::Serialize(ar);
	if (ar.IsStoring())
	{
		ar << m_strText << m_strURL << m_strLocation << m_strFriendlyName;

		ar << BOOL(m_pHlink!=NULL);
		if(m_pHlink!=NULL)
		{
			WriteObjectToStreamFlat(ar, m_pHlink);
		}

		CFont* pFont=GetFont();
		pFont->GetLogFont(&lfFont);
		ar.Write(&lfFont,sizeof(LOGFONT));

		ar << m_iHorAlignment << m_iVerAlignment << m_bSingleLine << m_rgbTextColour;
	}
	else
	{
		ar >> m_strText >> m_strURL >> m_strLocation >> m_strFriendlyName;

		BOOL bHlinkExists=FALSE;
		ar >> bHlinkExists;

		if(bHlinkExists)
		{
			IUnknownPtr pUnkHLink;
			ReadObjectFromStreamFlat(ar, &pUnkHLink);
			m_pHlink=pUnkHLink;
			m_dwSiteData=m_pDocument->GetNextSiteData();
			m_pHlink->SetHlinkSite(&m_pDocument->m_xHlinkSite,m_dwSiteData);
		}

		ar.Read(&lfFont,sizeof(LOGFONT));
		m_posFont=staticLoadedFontsList.GetFontPosIndirect(&lfFont);	
		ar >> m_iHorAlignment >> m_iVerAlignment >> m_bSingleLine >> m_rgbTextColour;
	}
}



void CDrawHLink::Draw(CDC* pDC)
{
	CDrawRect::Draw(pDC);
	CFont* pFont=GetFont();
	int nBackMode=pDC->SetBkMode(TRANSPARENT);
	CFont* pOldFont=pDC->SelectObject(pFont);
	COLORREF rgbOldColor=pDC->SetTextColor(m_rgbTextColour);

	UINT nFormat=DT_EXPANDTABS|DT_WORDBREAK|m_iHorAlignment;
	if(m_bSingleLine)
		nFormat|=m_iVerAlignment|DT_SINGLELINE;
	
	CRect rect=m_position;
	rect.DeflateRect(m_logpen.lopnWidth.x,m_logpen.lopnWidth.y);
	
	
	CString strTextToDraw=m_strText;
	pDC->DrawText(strTextToDraw,rect,nFormat);

	pDC->SelectObject(pOldFont);
	pDC->SetBkMode(nBackMode);
	pDC->SetTextColor(rgbOldColor);
}



CDrawObj* CDrawHLink::Clone(CDrawDoc* pDoc)
{
	CDrawHLink* pCopy=new CDrawHLink(m_position);
	
	pCopy->m_bPen = m_bPen;
	pCopy->m_logpen = m_logpen;
	pCopy->m_bBrush = m_bBrush;
	pCopy->m_logbrush = m_logbrush;
	pCopy->m_nShape = m_nShape;
	pCopy->m_roundness = m_roundness;
	
	pCopy->m_strText=m_strText;
	pCopy->m_strURL=m_strURL;
	pCopy->m_strLocation=m_strLocation;
	pCopy->m_strFriendlyName=m_strFriendlyName;
	
	pCopy->m_pHlink=NULL;

	if(m_pHlink)
	{
		pCopy->m_dwSiteData=m_pDocument->GetNextSiteData();
		HlinkClone(m_pHlink,
					IID_IHlink,
					&m_pDocument->m_xHlinkSite,
					pCopy->m_dwSiteData,
					(void**)&pCopy->m_pHlink);
	}
	
	pCopy->m_posFont=m_posFont;
	pCopy->m_iHorAlignment=m_iHorAlignment;
	pCopy->m_iVerAlignment=m_iVerAlignment;
	pCopy->m_bSingleLine=m_bSingleLine;
	pCopy->m_rgbTextColour=m_rgbTextColour;

	if (pDoc != NULL)
		pDoc->Add(pCopy);


	return pCopy;
}


void CDrawHLink::OnEditProperties()
{
	CPropertySheet sheet( _T("HLink Properties") );
	sheet.m_psh.dwFlags |= PSH_NOAPPLYNOW;
	
	CRectDlg dlg;
	dlg.m_bNoFill = !m_bBrush;
	dlg.m_penSize = m_bPen ? m_logpen.lopnWidth.x : 0;
	sheet.AddPage( &dlg );


	CHLinkPage page2;
	page2.m_strText=m_strText;
	page2.m_strURL=m_strURL;
	page2.m_strLocation=m_strLocation;
	page2.m_strFriendlyName=m_strFriendlyName;
	sheet.AddPage( &page2 );

	CHLinkFormatPage page3;
	CFont* pFont=GetFont();
	LOGFONT lf;
	pFont->GetLogFont(&lf);
	page3.SetLogFont(lf);
	page3.m_bSingleLine=m_bSingleLine;
	page3.m_iHorAlignment=m_iHorAlignment;
	page3.m_iVerAlignment=m_iVerAlignment;
	page3.m_rgbTextColour=m_rgbTextColour;
	sheet.AddPage( &page3 );

	if (sheet.DoModal() != IDOK)
		return;

	m_bBrush = !dlg.m_bNoFill;
	m_bPen = dlg.m_penSize > 0;
	if (m_bPen)
	{
		m_logpen.lopnWidth.x = dlg.m_penSize;
		m_logpen.lopnWidth.y = dlg.m_penSize;
	}


	m_strText=page2.m_strText;
	if(page2.IsLinkChanged())
	{
		DWORD dwSiteData=m_pDocument->GetNextSiteData();
		IHlinkPtr hlink=m_pDocument->CreateHlink(page2.m_strURL,
													page2.m_strLocation,
													page2.m_strFriendlyName,
													dwSiteData);

		if(hlink)
		{
			m_strURL=page2.m_strURL;
			m_strLocation=page2.m_strLocation;
			m_strFriendlyName=page2.m_strFriendlyName;
			m_dwSiteData=dwSiteData;
			m_pHlink=hlink;
		}
	}
	
	m_posFont=staticLoadedFontsList.GetFontPosIndirect(&page3.GetLogFont());	
	m_bSingleLine=page3.m_bSingleLine;
	m_iHorAlignment=page3.m_iHorAlignment;
	m_iVerAlignment=page3.m_iVerAlignment;
	m_rgbTextColour=page3.m_rgbTextColour;


	Invalidate();
	m_pDocument->SetModifiedFlag();
}


BOOL CDrawHLink::CreateFromData(COleDataObject& dataObject)
{
	USES_CONVERSION;
	TRY
	{
		m_dwSiteData=m_pDocument->GetNextSiteData();
		m_pHlink=m_pDocument->CreateHlinkFromData(&dataObject,m_dwSiteData);

		if(m_pHlink!=NULL)
		{
			LPWSTR pwzTarget=NULL,pwzLocation=NULL,pwzFriendly=NULL;
			m_pHlink->GetStringReference(HLINKGETREF_DEFAULT,&pwzTarget,&pwzLocation);
			m_pHlink->GetFriendlyName(HLFNAMEF_DEFAULT, &pwzFriendly);
			
			if(pwzTarget)
			{
				m_strURL=OLE2T(pwzTarget);
				CoTaskMemFree(pwzTarget);
			}

			if(pwzLocation)
			{
				m_strLocation=OLE2T(pwzLocation);
				CoTaskMemFree(pwzLocation);
			}

			if(pwzFriendly)
			{
				m_strFriendlyName=OLE2T(pwzFriendly);
				CoTaskMemFree(pwzFriendly);
			}

			m_strText=m_strFriendlyName;
			if(m_strText.IsEmpty())
				m_strText=m_strURL;
			
			return TRUE;
		}
	}
	CATCH_ALL(e)
	{
	}
	END_CATCH_ALL

	return FALSE;
}

void CDrawHLink::OnClick(CDrawView* pView)
{
	m_pDocument->NavigateToDrawHLink(this,pView);
}

BOOL CDrawHLink::OnSetCursor(CDrawView* pView, CPoint point)
{
	SetCursor(AfxGetApp()->LoadCursor(IDC_CURSOR_HLINK));
	return TRUE;
}

//HLink end